@@ -164,6 +164,30 @@ describe CarrierWave::ActiveRecord do
 
         expect(@event.reload.image).to be_blank
       end
+
+      context "with CarrierWave::MiniMagick" do
+        before(:each) do
+          @uploader.send(:include, CarrierWave::MiniMagick)
+        end
+
+        it "has width and height" do
+          @event.image = stub_file('landscape.jpg')
+          expect(@event.image.width).to eq 640
+          expect(@event.image.height).to eq 480
+        end
+      end
+
+      context "with CarrierWave::RMagick", :rmagick => true do
+        before(:each) do
+          @uploader.send(:include, CarrierWave::RMagick)
+        end
+
+        it "has width and height" do
+          @event.image = stub_file('landscape.jpg')
+          expect(@event.image.width).to eq 640
+          expect(@event.image.height).to eq 480
+        end
+      end
     end
 
     describe '#image=' do
@@ -472,7 +496,7 @@ describe CarrierWave::ActiveRecord do
 
     describe "#remote_image_url=" do
       before do
-        stub_request(:get, "www.example.com/test.jpg").to_return(body: File.read(file_path("test.jpg")))
+        stub_request(:get, "http://www.example.com/test.jpg").to_return(body: File.read(file_path("test.jpg")))
       end
 
       # FIXME ideally image_changed? and remote_image_url_changed? would return true
@@ -681,12 +705,10 @@ describe CarrierWave::ActiveRecord do
         expect(File.exist?(public_path('uploads/old.jpeg'))).to be_truthy
       end
 
-      pending do
-        it "should only delete the file once when the file is removed" do
-          @event.remove_image = true
-          expect_any_instance_of(CarrierWave::SanitizedFile).to receive(:delete).exactly(1).times
-          expect(@event.save).to be_truthy
-        end
+      pending("should only delete the file once when the file is removed") do
+        @event.remove_image = true
+        expect_any_instance_of(CarrierWave::SanitizedFile).to receive(:delete).exactly(1).times
+        expect(@event.save).to be_truthy
       end
     end
 
@@ -1193,7 +1215,7 @@ describe CarrierWave::ActiveRecord do
 
     describe "#remote_images_urls=" do
       before do
-        stub_request(:get, "www.example.com/test.jpg").to_return(body: File.read(file_path("test.jpg")))
+        stub_request(:get, "http://www.example.com/test.jpg").to_return(body: File.read(file_path("test.jpg")))
       end
 
       # FIXME ideally images_changed? and remote_images_urls_changed? would return true
