@@ -3,8 +3,10 @@ def fog_tests(fog_credentials)
     shared_examples_for "#{fog_credentials[:provider]} storage" do
 
       before do
+        WebMock.disable! unless Fog.mocking?
         CarrierWave.configure do |config|
           config.reset_config
+          config.fog_provider = "fog/#{fog_credentials[:provider].downcase}"
           config.fog_attributes = {}
           config.fog_credentials = fog_credentials
           config.fog_directory = CARRIERWAVE_DIRECTORY
@@ -33,6 +35,7 @@ end
         CarrierWave.configure do |config|
           config.reset_config
         end
+        WebMock.enable! unless Fog.mocking?
       end
 
       describe '#cache_stored_file!' do
@@ -43,175 +46,236 @@ end
         end
       end
 
-      describe '#store!' do
-
+      context '#acl_header' do
         let(:store_path) { 'uploads/test+.jpg' }
 
         before do
           allow(@uploader).to receive(:store_path).and_return(store_path)
-          @fog_file = @storage.store!(file)
         end
 
-        it "should upload the file" do
-          # reading the file after upload should return the body, not a closed tempfile
-          expect(@fog_file.read).to eq('this is stuff')
-          # make sure it actually uploaded to the service, too
-          expect(@directory.files.get(store_path).body).to eq('this is stuff')
-        end
-
-        it "should have a path" do
-          expect(@fog_file.path).to eq(store_path)
-        end
-
-        it "should have a content_type" do
-          expect(@fog_file.content_type).to eq(file.content_type)
-          expect(@directory.files.get(store_path).content_type).to eq(file.content_type)
-        end
-
-        it "should have an extension" do
-          expect(@fog_file.extension).to eq("jpg")
-        end
-
-        context "without asset_host" do
-          it "should have a public_url" do
-            unless fog_credentials[:provider] == 'Local'
-              expect(@fog_file.public_url).not_to be_nil
+        it 'includes acl_header when necessary' do
+          if file.is_a?(CarrierWave::Storage::Fog::File)
+            if @provider == 'AWS'
+              expect(@storage.connection).to receive(:copy_object)
+                                              .with(anything, anything, anything, anything, { "x-amz-acl"=>"public-read" }).and_call_original
+            else
+              expect(@storage.connection).to receive(:copy_object)
+                                              .with(anything, anything, anything, anything, {}).and_call_original
             end
           end
 
-          it "should have a url" do
-            unless fog_credentials[:provider] == 'Local'
-              expect(@fog_file.url).not_to be_nil
-            end
+          @storage.store!(file)
+        end
+      end
+
+      describe '#store!' do
+        let(:store_path) { 'uploads/test+.jpg' }
+
+        context 'when file is ::File' do
+          before do
+            allow(@uploader).to receive(:store_path).and_return(store_path)
+            @fog_file = @storage.store!(file)
           end
 
-          context "directory is a valid subdomain" do
-            before do
-              allow(@uploader).to receive(:fog_directory).and_return('assets.site.com')
+          it "should upload the file" do
+            # reading the file after upload should return the body, not a closed tempfile
+            expect(@fog_file.read).to eq('this is stuff')
+            # make sure it actually uploaded to the service, too
+            expect(@directory.files.get(store_path).body).to eq('this is stuff')
+          end
+
+          it "should have a path" do
+            expect(@fog_file.path).to eq(store_path)
+          end
+
+          it "should have a content_type" do
+            expect(@fog_file.content_type).to eq(file.content_type)
+            expect(@directory.files.get(store_path).content_type).to eq(file.content_type)
+          end
+
+          it "should have an extension" do
+            expect(@fog_file.extension).to eq("jpg")
+          end
+
+          context "without asset_host" do
+            it "should have a public_url" do
+              unless fog_credentials[:provider] == 'Local'
+                expect(@fog_file.public_url).not_to be_nil
+              end
             end
 
-            it "should use a subdomain URL for AWS" do
+            it "should have a url" do
+              unless fog_credentials[:provider] == 'Local'
+                expect(@fog_file.url).not_to be_nil
+              end
+            end
+
+            context "directory is a valid subdomain" do
+              before do
+                allow(@uploader).to receive(:fog_directory).and_return('assets-site-com')
+              end
+
+              it "should use a subdomain URL for AWS" do
+                if @provider == 'AWS'
+                  expect(@fog_file.public_url).to include('https://assets-site-com.s3.amazonaws.com')
+                end
+              end
+
+              it "should use accelerate domain if fog_aws_accelerate is true" do
+                if @provider == 'AWS'
+                  allow(@uploader).to receive(:fog_aws_accelerate).and_return(true)
+                  expect(@fog_file.public_url).to include('https://assets-site-com.s3-accelerate.amazonaws.com')
+                end
+              end
+            end
+
+            it "should not use a subdomain URL for AWS if the directory is not a valid subdomain" do
               if @provider == 'AWS'
-                expect(@fog_file.public_url).to include('https://assets.site.com.s3.amazonaws.com')
+                allow(@uploader).to receive(:fog_directory).and_return('SiteAssets')
+                expect(@fog_file.public_url).to include('https://s3.amazonaws.com/SiteAssets')
               end
             end
 
-            it "should use accelerate domain if fog_aws_accelerate is true" do
+            it "should not use a subdomain URL for AWS if https && the directory is not accessible over https as a virtual hosted bucket" do
               if @provider == 'AWS'
-                allow(@uploader).to receive(:fog_aws_accelerate).and_return(true)
-                expect(@fog_file.public_url).to include('https://assets.site.com.s3-accelerate.amazonaws.com')
+                allow(@uploader).to receive(:fog_use_ssl_for_aws).and_return(true)
+                allow(@uploader).to receive(:fog_directory).and_return('foo.bar')
+                expect(@fog_file.public_url).to include('https://s3.amazonaws.com/foo.bar')
+              end
+            end
+
+            it "should use a subdomain URL for AWS if http && the directory is not accessible over https as a virtual hosted bucket" do
+              if @provider == 'AWS'
+                allow(@uploader).to receive(:fog_use_ssl_for_aws).and_return(false)
+                allow(@uploader).to receive(:fog_directory).and_return('foo.bar')
+                expect(@fog_file.public_url).to include('http://foo.bar.s3.amazonaws.com/')
+              end
+            end
+
+            it "should use https as a default protocol" do
+              if @provider == 'AWS'
+                expect(@fog_file.public_url).to start_with 'https://'
+              end
+            end
+
+            it "should use https as a default protocol" do
+              if @provider == 'AWS'
+                allow(@uploader).to receive(:fog_use_ssl_for_aws).and_return(false)
+                expect(@fog_file.public_url).to start_with 'http://'
+              end
+            end
+
+            it "should use the google public url if available" do
+              if @provider == 'Google'
+                allow(@uploader).to receive(:fog_directory).and_return('SiteAssets')
+                expect(@fog_file.public_url).to include('https://storage.googleapis.com/SiteAssets')
               end
             end
           end
 
-          it "should not use a subdomain URL for AWS if the directory is not a valid subdomain" do
-            if @provider == 'AWS'
-              allow(@uploader).to receive(:fog_directory).and_return('SiteAssets')
-              expect(@fog_file.public_url).to include('https://s3.amazonaws.com/SiteAssets')
-            end
-          end
+          context "with asset_host" do
+            before { allow(@uploader).to receive(:asset_host).and_return(asset_host) }
 
-          it "should use https as a default protocol" do
-            if @provider == 'AWS'
-              expect(@fog_file.public_url).to start_with 'https://'
-            end
-          end
+            context "when a asset_host is a proc" do
 
-          it "should use https as a default protocol" do
-            if @provider == 'AWS'
-              allow(@uploader).to receive(:fog_use_ssl_for_aws).and_return(false)
-              expect(@fog_file.public_url).to start_with 'http://'
-            end
-          end
+              let(:asset_host) { proc { "http://foo.bar" } }
 
-          it "should use the google public url if available" do
-            if @provider == 'Google'
-              allow(@uploader).to receive(:fog_directory).and_return('SiteAssets')
-              expect(@fog_file.public_url).to include('https://storage.googleapis.com/SiteAssets')
-            end
-          end
-        end
+              describe "args passed to proc" do
+                let(:asset_host) { proc { |storage| expect(storage).to be_instance_of ::CarrierWave::Storage::Fog::File } }
 
-        context "with asset_host" do
-          before { allow(@uploader).to receive(:asset_host).and_return(asset_host) }
+                it "should be the uploader" do
+                  @fog_file.public_url
+                end
+              end
 
-          context "when a asset_host is a proc" do
+              it "should have a asset_host rooted public_url" do
+                expect(@fog_file.public_url).to eq('http://foo.bar/uploads/test%2B.jpg')
+              end
 
-            let(:asset_host) { proc { "http://foo.bar" } }
+              it "should have a asset_host rooted url" do
+                expect(@fog_file.url).to eq('http://foo.bar/uploads/test%2B.jpg')
+              end
 
-            describe "args passed to proc" do
-              let(:asset_host) { proc { |storage| expect(storage).to be_instance_of ::CarrierWave::Storage::Fog::File } }
+              it "should always have the same asset_host rooted url" do
+                expect(@fog_file.url).to eq('http://foo.bar/uploads/test%2B.jpg')
+                expect(@fog_file.url).to eq('http://foo.bar/uploads/test%2B.jpg')
+              end
 
-              it "should be the uploader" do
-                @fog_file.public_url
+              it 'should retrieve file name' do
+                expect(@fog_file.filename).to eq('test+.jpg')
               end
             end
 
-            it "should have a asset_host rooted public_url" do
-              expect(@fog_file.public_url).to eq('http://foo.bar/uploads/test%2B.jpg')
+            context "when a string" do
+              let(:asset_host) { "http://foo.bar" }
+
+              it "should have a asset_host rooted public_url" do
+                expect(@fog_file.public_url).to eq('http://foo.bar/uploads/test%2B.jpg')
+              end
+
+              it "should have a asset_host rooted url" do
+                expect(@fog_file.url).to eq('http://foo.bar/uploads/test%2B.jpg')
+              end
+
+              it "should always have the same asset_host rooted url" do
+                expect(@fog_file.url).to eq('http://foo.bar/uploads/test%2B.jpg')
+                expect(@fog_file.url).to eq('http://foo.bar/uploads/test%2B.jpg')
+              end
             end
 
-            it "should have a asset_host rooted url" do
-              expect(@fog_file.url).to eq('http://foo.bar/uploads/test%2B.jpg')
-            end
-
-            it "should always have the same asset_host rooted url" do
-              expect(@fog_file.url).to eq('http://foo.bar/uploads/test%2B.jpg')
-              expect(@fog_file.url).to eq('http://foo.bar/uploads/test%2B.jpg')
-            end
-
-            it 'should retrieve file name' do
-              expect(@fog_file.filename).to eq('test+.jpg')
-            end
           end
 
-          context "when a string" do
-            let(:asset_host) { "http://foo.bar" }
+          context "without extension" do
 
-            it "should have a asset_host rooted public_url" do
-              expect(@fog_file.public_url).to eq('http://foo.bar/uploads/test%2B.jpg')
+            let(:store_path) { 'uploads/test' }
+
+            it "should have no extension" do
+              expect(@fog_file.extension).to be_nil
             end
 
-            it "should have a asset_host rooted url" do
-              expect(@fog_file.url).to eq('http://foo.bar/uploads/test%2B.jpg')
+          end
+
+          it "should return filesize" do
+            expect(@fog_file.size).to eq(13)
+          end
+
+          it "should be deletable" do
+            @fog_file.delete
+            expect(@directory.files.head(store_path)).to eq(nil)
+          end
+
+          context "when the file has been deleted" do
+            before { @fog_file.delete }
+
+            it "should not error getting the file size" do
+              expect { @fog_file.size }.not_to raise_error
             end
 
-            it "should always have the same asset_host rooted url" do
-              expect(@fog_file.url).to eq('http://foo.bar/uploads/test%2B.jpg')
-              expect(@fog_file.url).to eq('http://foo.bar/uploads/test%2B.jpg')
+            it "should not error getting the content type" do
+              expect { @fog_file.content_type }.not_to raise_error
             end
           end
-
         end
 
-        context "without extension" do
-
-          let(:store_path) { 'uploads/test' }
-
-          it "should have no extension" do
-            expect(@fog_file.extension).to be_nil
+        context 'when file is ::StringIO' do
+          let(:file) do
+            CarrierWave::SanitizedFile.new(
+              :tempfile => StringIO.new('Test StringIO texts'),
+              :filename => 'test.jpg',
+              :content_type => 'image/jpeg'
+            )
           end
 
-        end
-
-        it "should return filesize" do
-          expect(@fog_file.size).to eq(13)
-        end
-
-        it "should be deletable" do
-          @fog_file.delete
-          expect(@directory.files.head(store_path)).to eq(nil)
-        end
-
-        context "when the file has been deleted" do
-          before { @fog_file.delete }
-
-          it "should not error getting the file size" do
-            expect { @fog_file.size }.not_to raise_error
+          before do
+            allow(@uploader).to receive(:store_path).and_return(store_path)
+            @fog_file = @storage.store!(file)
           end
 
-          it "should not error getting the content type" do
-            expect { @fog_file.content_type }.not_to raise_error
+          it "should upload the file" do
+            # reading the file after upload should return the body, not a closed tempfile
+            expect(@fog_file.read).to eq('Test StringIO texts')
+            # make sure it actually uploaded to the service, too
+            expect(@directory.files.get(store_path).body).to eq('Test StringIO texts')
           end
         end
       end
@@ -281,7 +345,7 @@ end
       end
 
       describe '#clean_cache!' do
-        let(:today) { '2016/10/09 10:00:00'.to_time }
+        let(:today) { Time.now.round }
         let(:five_days_ago) { today.ago(5.days) }
         let(:three_days_ago) { today.ago(3.days) }
         let(:yesterday) { today.yesterday }
@@ -369,12 +433,12 @@ end
 
           it "should not be available at public URL" do
             unless Fog.mocking? || fog_credentials[:provider] == 'Local'
-              expect(running{ open(@fog_file.public_url) }).to raise_error
+              expect(running{ open(@fog_file.public_url) }).to raise_error OpenURI::HTTPError
             end
           end
 
           it "should have an authenticated_url" do
-            if ['AWS', 'Rackspace', 'Google', 'OpenStack'].include?(@provider)
+            if ['AWS', 'Rackspace', 'Google', 'OpenStack', 'AzureRM'].include?(@provider)
               expect(@fog_file.authenticated_url).not_to be_nil
             end
           end
@@ -384,9 +448,24 @@ end
           end
 
           it "should handle query params" do
-            if @provider == 'AWS' && !Fog.mocking?
-              headers = Excon.get(@fog_file.url(:query => {"response-content-disposition" => "attachment"})).headers
-              expect(headers["Content-Disposition"]).to eq("attachment")
+            if ['AWS', 'Google'].include?(@provider)
+              url = @fog_file.url(:query => {"response-content-disposition" => "attachment"})
+              expect(url).to match(/response-content-disposition=attachment/)
+              unless Fog.mocking?
+                # Workaround for S3 SignatureDoesNotMatch issue
+                #   https://github.com/excon/excon/issues/475
+                Excon.defaults[:omit_default_port] = true
+                response = Excon.get(url)
+                expect(response.status).to be 200
+                expect(response.headers["Content-Disposition"]).to eq("attachment")
+              end
+            end
+          end
+
+          it "should not use #file to get signed url" do
+            if ['AWS', 'Google'].include?(@provider)
+              allow(@fog_file).to receive(:file).and_return(nil)
+              expect { @fog_file.url }.not_to raise_error
             end
           end
         end
